/***************************************************************************
 * $Id: tork.cpp,v 1.160 2007/12/30 12:58:22 hoganrobert Exp $
 *   Copyright (C) 2006 by Robert Hogan                                    *
 *   robert@roberthogan.net                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
 ***************************************************************************/
/* This file is part of the KDE project
   Copyright (c) 2004 Kevin Ottens <ervin ipsquad net>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef APPIMPL_H
#define APPIMPL_H

#include <kio/global.h>
#include <kio/job.h>
#include <kurl.h>
#include <dcopobject.h>

#include <tqobject.h>
#include <tqstring.h>
#include <tqdir.h>

class AppImpl : public TQObject
{
Q_OBJECT
public:
	AppImpl();

	void createTopLevelEntry(KIO::UDSEntry& entry) const;
	bool statByName(const TQString &filename, KIO::UDSEntry& entry);

	bool listRoot(TQValueList<KIO::UDSEntry> &list);

	bool parseURL(const KURL &url, TQString &name, TQString &path) const;
	bool realURL(const TQString &name, const TQString &path, KURL &url) const;

	int lastErrorCode() const { return m_lastErrorCode; }
	TQString lastErrorMessage() const { return m_lastErrorMessage; }
    bool listAppContents(const TQString &name, TQValueList<KIO::UDSEntry> &list);

private slots:
	KURL findBaseURL(const TQString &filename) const;
	void slotEntries(KIO::Job *job, const KIO::UDSEntryList &list);
	void slotResult(KIO::Job *job);
    void createManPageEntry(KIO::UDSEntry &entry,
                             const TQString &shortname);
    void createExeEntry(TQValueList<KIO::UDSEntry> &list,KIO::UDSEntry &entry,
                             const TQString &shortname,
                             const TQStringList &fullname);
    void createHomeDirEntry(TQValueList<KIO::UDSEntry> &list,KIO::UDSEntry &entry,
                             const TQString &shortname);
    void createKDEDataDirEntry(TQValueList<KIO::UDSEntry> &list,KIO::UDSEntry &entry,
                             const TQString &shortname);
    void createKDEConfigEntry(KIO::UDSEntry &entry,
                             const TQString &shortname);
    void createStandardConfigEntry(TQValueList<KIO::UDSEntry> &list, KIO::UDSEntry &entry,
                             const TQString &shortname);
    void createStandardDataDirEntry(TQValueList<KIO::UDSEntry> &list, KIO::UDSEntry &entry,
                             const TQString &shortname);
    void createTmpDirEntry(TQValueList<KIO::UDSEntry> &list, KIO::UDSEntry &entry,
                             const TQString &shortname);

    TQStringList getAppAddress(const TQString &name);
    TQStringList getFullLocation(const TQStringList & dirList, const TQString &name,
                            const TQDir::FilterSpec &filter, bool beginswith, bool recursive);
private:
	void createEntry(KIO::UDSEntry& entry, /*const TQString &directory,*/
	                 const TQString &file);

	bool m_lastListingEmpty;

	/// Last error code stored in class to simplify API.
	/// Note that this means almost no method can be const.
	int m_lastErrorCode;
	TQString m_lastErrorMessage;
};

#endif
